@@ -14,10 +14,10 @@ from .parsers import parse_prices, parse_loads, parse_generation, \
     parse_installed_capacity_per_plant, parse_crossborder_flows, \
     parse_unavailabilities, parse_contracted_reserve, parse_imbalance_prices_zip, \
     parse_netpositions, parse_procured_balancing_capacity
-from .decorators import retry, paginated, year_limited, day_limited
+from .decorators import retry, paginated, year_limited, day_limited, documents_limited
 
 __title__ = "entsoe-py"
-__version__ = "0.4.2"
+__version__ = "0.4.5"
 __author__ = "EnergieID.be"
 __license__ = "MIT"
 
@@ -109,6 +109,13 @@ class EntsoeRawClient:
                         f"The API is limited to {allowed} elements per "
                         f"request. This query requested for {requested} "
                         f"documents and cannot be fulfilled as is.")
+                elif 'requested data to be gathered via the offset parameter exceeds the allowed limit' in error_text:
+                    requested = error_text.split(' ')[-9]
+                    allowed = error_text.split(' ')[-30][:-2]
+                    raise PaginationError(
+                        f"The API is limited to {allowed} elements per "
+                        f"request. This query requested for {requested} "
+                        f"documents and cannot be fulfilled as is.")
             raise e
         else:
             # ENTSO-E has changed their server to also respond with 200 if there is no data but all parameters are valid
@@ -164,14 +171,15 @@ class EntsoeRawClient:
         response = self._base_request(params=params, start=start, end=end)
         return response.text
     
-    def query_net_position_dayahead(self, country_code: Union[Area, str],
-                            start: pd.Timestamp, end: pd.Timestamp) -> str:
+    def query_net_position(self, country_code: Union[Area, str],
+                           start: pd.Timestamp, end: pd.Timestamp, dayahead: bool = True) -> str:
         """
         Parameters
         ----------
         country_code : Area|str
         start : pd.Timestamp
         end : pd.Timestamp
+        dayahead : bool
 
         Returns
         -------
@@ -185,6 +193,9 @@ class EntsoeRawClient:
             'in_Domain': area.code,
             'out_Domain': area.code
         }
+        if not dayahead:
+            params.update({'Contract_MarketAgreement.Type': "A07"})
+
         response = self._base_request(params=params, start=start, end=end)
         return response.text
 
@@ -554,13 +565,46 @@ class EntsoeRawClient:
             doctype="A31", contract_marketagreement_type="A07",
             auction_type=("A01" if implicit==True else "A02"))
 
+    def query_offered_capacity(
+        self, country_code_from: Union[Area, str],
+            country_code_to: Union[Area, str], start: pd.Timestamp,
+            end: pd.Timestamp, contract_marketagreement_type: str,
+            implicit:bool = True,**kwargs) -> str:
+        """
+        Allocated result documents, for OC evolution see query_intraday_offered_capacity
+
+        Parameters
+        ----------
+        country_code_from : Area|str
+        country_code_to : Area|str
+        start : pd.Timestamp
+        end : pd.Timestamp
+        contract_marketagreement_type : str
+            type of contract (see mappings.MARKETAGREEMENTTYPE)
+        implicit: bool (True = implicit - default for most borders. False = explicit - for instance BE-GB)
+
+        Returns
+        -------
+        str
+        """
+        if implicit:
+            business_type = None
+        else:
+            business_type = "B05"
+        return self._query_crossborder(
+            country_code_from=country_code_from,
+            country_code_to=country_code_to, start=start, end=end,
+            doctype=("A31" if implicit else "A25"),
+            contract_marketagreement_type=contract_marketagreement_type,
+            auction_type=("A01" if implicit else "A02"),
+            business_type=business_type)
 
     def _query_crossborder(
             self, country_code_from: Union[Area, str],
             country_code_to: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, doctype: str,
             contract_marketagreement_type: Optional[str] = None,
-            auction_type: Optional[str] = None) -> str:
+            auction_type: Optional[str] = None, business_type: Optional[str] = None) -> str:
         """
         Generic function called by query_crossborder_flows, 
         query_scheduled_exchanges, query_net_transfer_capacity_DA/WA/MA/YA and query_.
@@ -573,6 +617,7 @@ class EntsoeRawClient:
         end : pd.Timestamp
         doctype: str
         contract_marketagreement_type: str
+        business_type: str
 
         Returns
         -------
@@ -592,6 +637,9 @@ class EntsoeRawClient:
         if auction_type is not None:
             params[
                 'Auction.Type'] = auction_type
+        if business_type is not None:
+            params[
+                'businessType'] = business_type
 
         response = self._base_request(params=params, start=start, end=end)
         return response.text
@@ -690,7 +738,8 @@ class EntsoeRawClient:
     def query_contracted_reserve_prices(
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, type_marketagreement_type: str,
-            psr_type: Optional[str] = None) -> str:
+            psr_type: Optional[str] = None,
+            offset: int = 0) -> str:
         """
         Parameters
         ----------
@@ -701,6 +750,8 @@ class EntsoeRawClient:
             type of contract (see mappings.MARKETAGREEMENTTYPE)
         psr_type : str
             filter query for a specific psr type
+        offset : int
+            offset for querying more than 100 documents
 
         Returns
         -------
@@ -711,6 +762,7 @@ class EntsoeRawClient:
             'documentType': 'A89',
             'controlArea_Domain': area.code,
             'type_MarketAgreement.Type': type_marketagreement_type,
+            'offset': offset
         }
         if psr_type:
             params.update({'psrType': psr_type})
@@ -720,7 +772,8 @@ class EntsoeRawClient:
     def query_contracted_reserve_amount(
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, type_marketagreement_type: str,
-            psr_type: Optional[str] = None) -> str:
+            psr_type: Optional[str] = None,
+            offset: int = 0) -> str:
         """
         Parameters
         ----------
@@ -731,6 +784,8 @@ class EntsoeRawClient:
             type of contract (see mappings.MARKETAGREEMENTTYPE)
         psr_type : str
             filter query for a specific psr type
+        offset : int
+            offset for querying more than 100 documents
 
         Returns
         -------
@@ -741,6 +796,7 @@ class EntsoeRawClient:
             'documentType': 'A81',
             'controlArea_Domain': area.code,
             'type_MarketAgreement.Type': type_marketagreement_type,
+            'offset': offset
         }
         if psr_type:
             params.update({'psrType': psr_type})
@@ -751,7 +807,8 @@ class EntsoeRawClient:
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, doctype: str, docstatus: Optional[str] = None,
             periodstartupdate: Optional[pd.Timestamp] = None,
-            periodendupdate: Optional[pd.Timestamp] = None) -> bytes:
+            periodendupdate: Optional[pd.Timestamp] = None,
+            offset: int = 0) -> bytes:
         """
         Generic unavailibility query method.
         This endpoint serves ZIP files.
@@ -766,6 +823,7 @@ class EntsoeRawClient:
         docstatus : str, optional
         periodstartupdate : pd.Timestamp, optional
         periodendupdate : pd.Timestamp, optional
+        offset : int
 
         Returns
         -------
@@ -774,7 +832,8 @@ class EntsoeRawClient:
         area = lookup_area(country_code)
         params = {
             'documentType': doctype,
-            'biddingZone_domain': area.code
+            'biddingZone_domain': area.code,
+            'offset': offset
             # ,'businessType': 'A53 (unplanned) | A54 (planned)'
         }
         if docstatus:
@@ -790,7 +849,8 @@ class EntsoeRawClient:
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, docstatus: Optional[str] = None,
             periodstartupdate: Optional[pd.Timestamp] = None,
-            periodendupdate: Optional[pd.Timestamp] = None) -> bytes:
+            periodendupdate: Optional[pd.Timestamp] = None,
+            offset: int = 0) -> bytes:
         """
         This endpoint serves ZIP files.
         The query is limited to 200 items per request.
@@ -803,6 +863,8 @@ class EntsoeRawClient:
         docstatus : str, optional
         periodstartupdate : pd.Timestamp, optional
         periodendupdate : pd.Timestamp, optional
+        offset : int
+
 
         Returns
         -------
@@ -811,7 +873,7 @@ class EntsoeRawClient:
         content = self._query_unavailability(
             country_code=country_code, start=start, end=end, doctype="A80",
             docstatus=docstatus, periodstartupdate=periodstartupdate,
-            periodendupdate=periodendupdate)
+            periodendupdate=periodendupdate, offset=offset)
         return content
 
     def query_unavailability_of_production_units(
@@ -847,7 +909,9 @@ class EntsoeRawClient:
             country_code_to: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, docstatus: Optional[str] = None,
             periodstartupdate: Optional[pd.Timestamp] = None,
-            periodendupdate: Optional[pd.Timestamp] = None, **kwargs) -> bytes:
+            periodendupdate: Optional[pd.Timestamp] = None,
+            offset: int = 0,
+            **kwargs) -> bytes:
         """
         Generic unavailibility query method.
         This endpoint serves ZIP files.
@@ -862,6 +926,7 @@ class EntsoeRawClient:
         docstatus : str, optional
         periodstartupdate : pd.Timestamp, optional
         periodendupdate : pd.Timestamp, optional
+        offset : int
 
         Returns
         -------
@@ -872,7 +937,8 @@ class EntsoeRawClient:
         params = {
             'documentType': "A78",
             'in_Domain': area_in.code,
-            'out_Domain': area_out.code
+            'out_Domain': area_out.code,
+            'offset': offset
         }
         if docstatus:
             params['docStatus'] = docstatus
@@ -904,8 +970,8 @@ class EntsoeRawClient:
 
 class EntsoePandasClient(EntsoeRawClient):
     @year_limited
-    def query_net_position_dayahead(self, country_code: Union[Area, str],
-                            start: pd.Timestamp, end: pd.Timestamp) -> pd.Series:
+    def query_net_position(self, country_code: Union[Area, str],
+                            start: pd.Timestamp, end: pd.Timestamp, dayahead: bool = True) -> pd.Series:
         """
 
         Parameters
@@ -919,8 +985,8 @@ class EntsoePandasClient(EntsoeRawClient):
 
         """
         area = lookup_area(country_code)
-        text = super(EntsoePandasClient, self).query_net_position_dayahead(
-            country_code=area, start=start, end=end)
+        text = super(EntsoePandasClient, self).query_net_position(
+            country_code=area, start=start, end=end, dayahead=dayahead)
         series = parse_netpositions(text)
         series = series.tz_convert(area.tz)
         series = series.truncate(before=start, after=end)
@@ -1375,6 +1441,43 @@ class EntsoePandasClient(EntsoeRawClient):
         ts = ts.tz_convert(area_from.tz)
         ts = ts.truncate(before=start, after=end)
         return ts
+
+    @year_limited
+    def query_offered_capacity(
+        self, country_code_from: Union[Area, str],
+            country_code_to: Union[Area, str], start: pd.Timestamp,
+            end: pd.Timestamp, contract_marketagreement_type: str,
+            implicit:bool = True,**kwargs) -> pd.Series:
+        """
+        Allocated result documents, for OC evolution see query_intraday_offered_capacity
+        Note: Result will be in the timezone of the origin country  --> to check
+
+        Parameters
+        ----------
+        country_code_from : Area|str
+        country_code_to : Area|str
+        start : pd.Timestamp
+        end : pd.Timestamp
+        contract_marketagreement_type : str
+            type of contract (see mappings.MARKETAGREEMENTTYPE)
+        implicit: bool (True = implicit - default for most borders. False = explicit - for instance BE-GB)
+        Returns
+        -------
+        pd.Series
+        """
+        area_to = lookup_area(country_code_to)
+        area_from = lookup_area(country_code_from)
+        text = super(EntsoePandasClient, self).query_offered_capacity(
+            country_code_from=area_from,
+            country_code_to=area_to,
+            start=start,
+            end=end,
+            contract_marketagreement_type=contract_marketagreement_type,
+            implicit=implicit)
+        ts = parse_crossborder_flows(text)
+        ts = ts.tz_convert(area_from.tz)
+        ts = ts.truncate(before=start, after=end)
+        return ts
     
     @year_limited
     def query_imbalance_prices(
@@ -1464,10 +1567,12 @@ class EntsoePandasClient(EntsoeRawClient):
     
     @year_limited
     @paginated
+    @documents_limited(100)
     def query_contracted_reserve_prices(
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, type_marketagreement_type: str,
-            psr_type: Optional[str] = None) -> pd.DataFrame:
+            psr_type: Optional[str] = None,
+            offset: int = 0) -> pd.DataFrame:
         """
         Parameters
         ----------
@@ -1478,6 +1583,8 @@ class EntsoePandasClient(EntsoeRawClient):
             type of contract (see mappings.MARKETAGREEMENTTYPE)
         psr_type : str
             filter query for a specific psr type
+        offset : int
+            offset for querying more than 100 documents
 
         Returns
         -------
@@ -1487,7 +1594,7 @@ class EntsoePandasClient(EntsoeRawClient):
         text = super(EntsoePandasClient, self).query_contracted_reserve_prices(
             country_code=area, start=start, end=end,
             type_marketagreement_type=type_marketagreement_type,
-            psr_type=psr_type)
+            psr_type=psr_type, offset=offset)
         df = parse_contracted_reserve(text, area.tz, "procurement_price.amount")
         df = df.tz_convert(area.tz)
         df = df.truncate(before=start, after=end)
@@ -1495,10 +1602,12 @@ class EntsoePandasClient(EntsoeRawClient):
 
     @year_limited
     @paginated
+    @documents_limited(100)
     def query_contracted_reserve_amount(
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, type_marketagreement_type: str,
-            psr_type: Optional[str] = None) -> pd.DataFrame:
+            psr_type: Optional[str] = None,
+            offset: int = 0) -> pd.DataFrame:
         """
         Parameters
         ----------
@@ -1509,6 +1618,8 @@ class EntsoePandasClient(EntsoeRawClient):
             type of contract (see mappings.MARKETAGREEMENTTYPE)
         psr_type : str
             filter query for a specific psr type
+        offset : int
+            offset for querying more than 100 documents
 
         Returns
         -------
@@ -1518,7 +1629,7 @@ class EntsoePandasClient(EntsoeRawClient):
         text = super(EntsoePandasClient, self).query_contracted_reserve_amount(
             country_code=area, start=start, end=end,
             type_marketagreement_type=type_marketagreement_type,
-            psr_type=psr_type)
+            psr_type=psr_type, offset=offset)
         df = parse_contracted_reserve(text, area.tz, "quantity")
         df = df.tz_convert(area.tz)
         df = df.truncate(before=start, after=end)
@@ -1526,11 +1637,13 @@ class EntsoePandasClient(EntsoeRawClient):
 
     @year_limited
     @paginated
+    @documents_limited(200)
     def _query_unavailability(
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, doctype: str, docstatus: Optional[str] = None,
             periodstartupdate: Optional[pd.Timestamp] = None,
-            periodendupdate: Optional[pd.Timestamp] = None) -> pd.DataFrame:
+            periodendupdate: Optional[pd.Timestamp] = None,
+            offset: int = 0) -> pd.DataFrame:
         """
         Parameters
         ----------
@@ -1541,6 +1654,7 @@ class EntsoePandasClient(EntsoeRawClient):
         docstatus : str, optional
         periodstartupdate : pd.Timestamp, optional
         periodendupdate : pd.Timestamp, optional
+        offset : int
 
         Returns
         -------
@@ -1550,12 +1664,12 @@ class EntsoePandasClient(EntsoeRawClient):
         content = super(EntsoePandasClient, self)._query_unavailability(
             country_code=area, start=start, end=end, doctype=doctype,
             docstatus=docstatus, periodstartupdate=periodstartupdate,
-            periodendupdate=periodendupdate)
+            periodendupdate=periodendupdate, offset=offset)
         df = parse_unavailabilities(content, doctype)
         df = df.tz_convert(area.tz)
         df['start'] = df['start'].apply(lambda x: x.tz_convert(area.tz))
         df['end'] = df['end'].apply(lambda x: x.tz_convert(area.tz))
-        df = df.truncate(before=start, after=end)
+        df = df[(df['start'] < end) | (df['end'] > start)]
         return df
 
     def query_unavailability_of_generation_units(
@@ -1615,6 +1729,7 @@ class EntsoePandasClient(EntsoeRawClient):
             end: pd.Timestamp, docstatus: Optional[str] = None,
             periodstartupdate: Optional[pd.Timestamp] = None,
             periodendupdate: Optional[pd.Timestamp] = None,
+            offset: int = 0,
             **kwargs) -> pd.DataFrame:
         """
         Parameters
@@ -1626,6 +1741,7 @@ class EntsoePandasClient(EntsoeRawClient):
         docstatus : str, optional
         periodstartupdate : pd.Timestamp, optional
         periodendupdate : pd.Timestamp, optional
+        offset : int
 
         Returns
         -------
@@ -1636,12 +1752,12 @@ class EntsoePandasClient(EntsoeRawClient):
         content = super(EntsoePandasClient,
                         self).query_unavailability_transmission(
             area_from, area_to, start, end, docstatus, periodstartupdate,
-            periodendupdate)
+            periodendupdate, offset=offset)
         df = parse_unavailabilities(content, "A78")
         df = df.tz_convert(area_from.tz)
         df['start'] = df['start'].apply(lambda x: x.tz_convert(area_from.tz))
         df['end'] = df['end'].apply(lambda x: x.tz_convert(area_from.tz))
-        df = df.truncate(before=start, after=end)
+        df = df[(df['start'] < end) | (df['end'] > start)]
         return df
 
     def query_withdrawn_unavailability_of_generation_units(
@@ -1660,7 +1776,7 @@ class EntsoePandasClient(EntsoeRawClient):
         """
         df = self.query_unavailability_of_generation_units(
             country_code=country_code, start=start, end=end, docstatus='A13')
-        df = df.truncate(before=start, after=end)
+        df = df[(df['start'] < end) | (df['end'] > start)]
         return df
 
     @day_limited
